import {createStore} from 'vuex'
import moment from "moment";
import {Seasons, SeasonService} from "@/services/SeasonService";
import {ItemTypes} from "@/data/ItemTypes";
import createPersistedState from "vuex-persistedstate";
import ItemService from "@/services/ItemService";
import iziToast from "izitoast";
import AnimalService from "@/services/AnimalService";
import {CropAssetsDefault, CropAssetsWinter} from "@/data/Paths";


export default createStore({
    state: {
        initialized: null,
        tickBeat: false,
        player: {
            // TODO: ADD PLAYER INTRODUCTION
            name: 'Franz',
            money: 1_000,
            unlocked: {
                fields: 48
            },
        },
        time: {
            stamp: null,
            week: null,
            season: null,
            weekday: null,
            weekdayNr: null
        },
        inventory: [],
        inventorySortDesc: true,
        inventorySortQntyDesc: true,
        groupInventory: false,
        cropField: [],
        cropAssetFolder: 'default',
        selections: {
            field: null,
            item: null
        },
        animals: [
            {
                id: 1,
                name: 'Nugget',
                position: {
                    facing: 'left',
                    moving: false,
                    x: 50,
                    y: 50
                }
            }
        ],
    },
    getters: {
        selectedField(state) {
            return state.cropField.find(function (field) {
                return field.id === state.selections.field;
            }) ?? {};
        },
        getInventoryWithItemData(state) {
            return state.inventory.filter((inventoryItem) => {
                return inventoryItem.quantity > 0;
            }).map((inventoryItem) => {
                let item = ItemService.getItemByID(inventoryItem.id);

                item.quantity = inventoryItem.quantity;
                return item;
            }) ?? [];
        },
        getSeedsInInventory(state) {
            return state.inventory.filter((inventoryItem) => {
                return inventoryItem.type === ItemTypes.Seeds && inventoryItem.quantity > 0;
            }).map((inventoryItem) => {
                let item = ItemService.getItemByID(inventoryItem.id);

                item.quantity = inventoryItem.quantity;
                return item;
            }) ?? [];
        },
        getFieldProductsInInventory(state) {
            return state.inventory.filter((inventoryItem) => {
                return inventoryItem.type === ItemTypes.FieldProducts && inventoryItem.quantity > 0;
            }).map((inventoryItem) => {
                let item = ItemService.getItemByID(inventoryItem.id);

                item.quantity = inventoryItem.quantity;
                return item;
            }) ?? [];
        },
        getAnimalProductsInInventory(state) {
            return state.inventory.filter((inventoryItem) => {
                return inventoryItem.type === ItemTypes.AnimalProducts && inventoryItem.quantity > 0;
            }).map((inventoryItem) => {
                let item = ItemService.getItemByID(inventoryItem.id);

                item.quantity = inventoryItem.quantity;
                return item;
            }) ?? [];
        },
        getPreparedMealsInInventory(state) {
            return state.inventory.filter((inventoryItem) => {
                return inventoryItem.type === ItemTypes.PreparedMeals && inventoryItem.quantity > 0;
            }).map((inventoryItem) => {
                let item = ItemService.getItemByID(inventoryItem.id);

                item.quantity = inventoryItem.quantity;
                return item;
            }) ?? [];
        },
        getAnimalsWithAnimalData(state) {
            return state.animals.map((animal) => {
                animal.data = AnimalService.getAnimalByID(animal.id)
                return animal;
            });
        }
    },
    mutations: {
        initialize(state) {
            this.commit('checkForSeasonChange');

            state.time.week = moment().week();
            state.time.season = SeasonService.getSeasonByWeek(state.time.week);

            if (state.time.season === Seasons.Winter) {
                state.cropAssetFolder = CropAssetsWinter;
            } else {
                state.cropAssetFolder = CropAssetsDefault;
            }

            state.time.weekday = moment().weekday();
            state.time.weekdayNr = moment().format('dddd');
            state.initialized = moment();

            this.commit('generateField');
        },
        updateTimestamp(state) {
            state.tickBeat = !state.tickBeat;
            state.time.stamp = moment().format('H:mm:ss')
        },
        generateField(state) {
            let fieldIndex = 0;

            while (state.cropField.length < state.player.unlocked.fields) {
                fieldIndex++;

                state.cropField.push({
                    id: fieldIndex
                });
            }
        },
        selectField(state, id) {
            console.debug('select field with id ' + id)
            state.selections.field = id;
        },
        plantSeed(state, seed) {
            let selectedField = state.cropField.find((field) => {
                return field.id === state.selections.field;
            });

            /*** SEED PLANTING ***/
            selectedField.data = {
                planted: moment(),
                readyForHarvest: moment().add(seed.cropData.timeToGrow, 'minutes'),
                percentDone: 0,
                canBeHarvested: false,
                seed_id: seed.id
            }

            this.commit('removeItemFromInventory', {
                item: seed,
                quantity: 1
            });
        },
        harvestSeed(state, field) {
            let seed = ItemService.getItemByID(field.data.seed_id);
            let productFromSeed = ItemService.getItemByID(seed.cropData.product_id);

            if (field.data.percentDone < 100 || !field.data.readyForHarvest) {
                // bloody cheater
                return false;
            }

            iziToast.success({
                title: 'Harvested '+productFromSeed.name,
                image: productFromSeed.icon
            });

            this.commit('addItemToInventory', {
                item: productFromSeed,
                quantity: 1
            });

            field.data = null;
        },
        checkForSeasonChange(state) {
            let prevDate = state.initialized;

            if (moment(prevDate).weekday() === 7 && moment().weekday() === 1) {
                console.debug('SEASON CHANGE DETECTED');

                // TODO: KILL ALL THE CROPS
            }
        },
        buyItem(state, props = {item: Object, quantity: Number}) {
            let purchasePrice = props.item.economy.buy * props.quantity;

            if (purchasePrice > state.player.money) {
                iziToast.error({
                    title: "Not enough money to buy " + props.quantity + " of " + props.item.name
                });
                return false;
            }

            this.commit('addItemToInventory', {
                item: props.item,
                quantity: props.quantity
            });

            state.player.money -= purchasePrice;

            iziToast.success({
                title: 'Purchased ' + props.item.name + (props.quantity > 1 ? ' (' + props.quantity + ')' : '')
            });
        },
        sellItem(state, props = {item: Object, quantity: Number}) {
            let sellPrice = props.item.economy.sell * props.quantity;

            if (props.item.quantity < props.quantity) {
                iziToast.error({
                    title: "You only have "+props.item.quantity+" of "+props.item.name+", but you're trying to sell "+props.quantity
                });
                return false;
            }

            this.commit('removeItemFromInventory', {
                item: props.item,
                quantity: props.quantity
            });

            state.player.money = Number(state.player.money) + Number(sellPrice);

            iziToast.success({
                title: 'Sold ' + (props.quantity > 1 ? props.quantity + " of " : '') + props.item.name + " for " + sellPrice + "$",
                image: props.item.icon
            });
        },
        addItemToInventory(state, props = {item: Object, quantity: Number}) {
            let itemInInventory = state.inventory.find((inventoryItem) => {
                return inventoryItem.id === props.item.id;
            })

            if (itemInInventory) {
                itemInInventory.quantity = Number(itemInInventory.quantity) + Number(props.quantity);
            } else {
                /*** INVENTORY ITEM CREATION ***/
                state.inventory.push({
                    id: props.item.id,
                    type: props.item.type,
                    quantity: props.quantity
                })
            }
        },
        removeItemFromInventory(state, props = {item: Object, quantity: Number}) {
            let itemInInventory = state.inventory.find((inventoryItem) => {
                return inventoryItem.id === props.item.id;
            });

            if (itemInInventory) {
                itemInInventory.quantity = Number(itemInInventory.quantity) - Number(props.quantity);
            }
        },
        updateCrops(state) {
            state.cropField.filter((field) => {
                return field.data != null && !field.data.canBeHarvested;
            }).forEach(function (field) {
                let seed = ItemService.getItemByID(field.data.seed_id);

                if (field.data.readyForHarvest < moment()) {
                    field.data.canBeHarvested = true;
                }

                if (field.data.percentDone <= 100) {
                    let minutesSincePlanting = moment.duration(moment().diff(field.data.planted)).asMinutes();
                    field.data.percentDone = Number(
                    Number(minutesSincePlanting) / Number(seed.cropData.timeToGrow) * 100
                    ).toFixed(2);

                    if (field.data.percentDone >= 100) {
                        field.data.percentDone = 100;
                        field.data.canBeHarvested = true;
                    }
                } else {
                    field.data.canBeHarvested = true;
                }
            });
        },
        alphaAddMoney(state) {
            state.player.money = Number(state.player.money) + Number(1_000);
        },
        sortInventory(state) {
            state.inventory = state.inventory.sort((a, b) => {
                let aItem = ItemService.getItemByID(a.id);
                let bItem = ItemService.getItemByID(b.id);

                if (state.inventorySortDesc === true) {
                    return (bItem.name > aItem.name) ? 1 : -1;
                } else {
                    return (aItem.name < bItem.name) ? -1 : 1;
                }
            })

            state.inventorySortDesc = !state.inventorySortDesc;
        },
        sortInventoryByQuantity(state) {
            state.inventory = state.inventory.sort((a, b) => {
                if (state.inventorySortQntyDesc === true) {
                    return (b.quantity > a.quantity) ? 1 : -1;
                } else {
                    return (a.quantity < b.quantity) ? -1 : 1;
                }
            });

            state.inventorySortQntyDesc = !state.inventorySortQntyDesc;
        },
        toggleGrouping(state) {
            state.groupInventory = !state.groupInventory;
        },
        updateAnimals(state) {
            state.animals.filter((animal) => {
                return !animal.production || !animal.production.canBeCollected || !animal.production.percent >= 100;
            }).forEach(function (animal) {
                let animalData = AnimalService.getAnimalByID(animal.id);
                if (!animal.production) {
                    animal.production = {
                        started: moment(),
                        ready: moment().add(animalData.production.time, 'minutes'),
                        percent: 0,
                        canBeCollected: false
                    }
                }

                if (animal.production.ready < moment()) {
                    animal.production.canBeCollected = true;
                }

                if (animal.production.percent <= 100) {
                    let minutesSinceLastCollection = moment.duration(moment().diff(animal.production.started)).asMinutes();
                    animal.production.percent = Number(
                    Number(minutesSinceLastCollection) / Number(animalData.production.time) * 100
                    ).toFixed(2);

                    if (animal.production.percent >= 100) {
                        animal.production.percent = 100;
                        animal.production.canBeCollected = true;
                    }
                } else {
                    animal.production.canBeCollected = true;
                }
            });
        },
        doAnimalMovement(state) {
            state.animals.forEach((animal) => {
                animal.position.moving = false;
                let moveChance = Math.random();
                // console.log(moveChance)
                // console.log(animal.position.x+'|'+animal.position.y)

                if (moveChance > 0.6) {
                    animal.position.moving = true;
                    let randomX = Math.random() * 5 - 2.0;
                    let randomY = Math.random() * 5 - 2.0;
                    // console.log(randomX+'|'+randomY)
                    let newX = Math.floor(Number(animal.position.x) + Number(randomX));
                    let newY = Math.floor(Number(animal.position.y) + Number(randomY));



                    animal.position.x = (newX > 0 ? (newX < 60 ? newX: 60) : 0);
                    animal.position.y = (newY > 0 ? (newY < 70 ? newY: 70) : 0);

                    animal.position.facing = randomY > 0 ? 'right' : 'left';
                    // console.debug('lazy chicken did move')
                } else {
                    // console.debug('lazy chicken did not move')
                }

                setTimeout(() => {
                    animal.position.moving = false;
                }, 500)
            });
        }
    },
    actions: {},
    modules: {},
    plugins: [createPersistedState()],
});
